@@ -3,20 +3,20 @@
  * License Version 1.1 (the "MPL"); you may not use this file
  * except in compliance with the MPL. You may obtain a copy of
  * the MPL at http://www.mozilla.org/MPL/
- * 
+ *
  * Software distributed under the MPL is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the MPL for the specific language governing
  * rights and limitations under the MPL.
- * 
+ *
  * The Original Code is protoZilla.
- * 
+ *
  * The Initial Developer of the Original Code is Ramalingam Saravanan.
  * Portions created by Ramalingam Saravanan <svn@xmlterm.org> are
  * Copyright (C) 2000 Ramalingam Saravanan. All Rights Reserved.
- * 
+ *
  * Contributor(s):
- * 
+ *
  * Alternatively, the contents of this file may be used under the
  * terms of the GNU General Public License (the "GPL"), in which case
  * the provisions of the GPL are applicable instead of
@@ -30,7 +30,7 @@
  * GPL.
  */
 
-// Logging of debug output 
+// Logging of debug output
 // The following define statement should occur before any include statements
 #define FORCE_PR_LOG       /* Allow logging even in release build */
 
@@ -124,7 +124,7 @@ nsIPCService::Init()
   rv = mConsole->Open(500, 80, PR_FALSE);
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIObserverService> observerSvc = 
+  nsCOMPtr<nsIObserverService> observerSvc =
            do_GetService("@mozilla.org/observer-service;1");
 
   if (observerSvc) {
@@ -150,7 +150,7 @@ nsIPCService::Shutdown()
 
   IPC_Shutdown();
 
-  nsCOMPtr<nsIObserverService> observerSvc = 
+  nsCOMPtr<nsIObserverService> observerSvc =
            do_GetService("@mozilla.org/observer-service;1");
 
   if (observerSvc) {
@@ -443,7 +443,7 @@ NS_IMETHODIMP
 nsIPCService::ExecAsync(const char* command,
                         PRBool useShell,
                         const char* preInput,
-                        const char* inputData, PRUint32 inputLength, 
+                        const char* inputData, PRUint32 inputLength,
                         const char** env, PRUint32 envCount,
                         nsIPipeListener* outConsole,
                         nsIPipeListener* errConsole,
@@ -624,7 +624,8 @@ nsIPCService::GetCookie(char **_retval)
 
 
 NS_IMETHODIMP
-nsIPCService::NewStringChannel(nsIURI* aURI, const nsACString &aContentType,                                  const nsACString &aContentCharset,
+nsIPCService::NewStringChannel(nsIURI* aURI, const nsACString &aContentType,
+                               const nsACString &aContentCharset,
                                const char *aData, nsIChannel **result)
 {
   nsresult rv;
