@@ -53,9 +53,14 @@
 #include "nsString.h"
 #include "nsIPrompt.h"
 #include "nsNetUtil.h"
-#include "nsFileStream.h"
 #include "nsIThread.h"
 #include "nsIFactory.h"
+#ifdef _ENIG_MOZILLA_1_8
+#include "nsIFileStream.h"
+#include "nsMsgComposeStringBundle.h"
+#else
+#include "nsComposeStrings.h"
+#endif
 #undef MOZILLA_INTERNAL_API
 
 #ifdef PR_LOGGING
@@ -445,6 +450,29 @@ nsEnigMsgCompose::Init()
   nsCOMPtr<nsIEnigmail> enigmailSvc = do_GetService(NS_ENIGMAIL_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
+  if (usePgpMime && signMsg && (! encryptMsg)) {
+    // determine hash algorithm to use for PGP/MIME signed msg
+    PRInt32 exitCode;
+    PRUnichar* ha;
+
+    rv = enigmailSvc->DetermineHashAlgorithm(prompter,
+                                             mUIFlags,
+                                             mSenderEmailAddr.get(),
+                                             &ha,
+                                             &exitCode);
+
+    DEBUG_LOG(("nsEnigMsgCompose::Init: DetermineHash: rv=%d, exitCode=%d\n", rv, exitCode));
+
+    if (NS_FAILED(rv))
+      return rv;
+
+    if (exitCode != 0)
+      return NS_ERROR_SMTP_PASSWORD_UNDEFINED;
+
+    mHashAlgorithm = NS_ConvertUTF16toUTF8(ha).get();
+    DEBUG_LOG(("nsEnigMsgCompose::Init: hashAlgorithm=%s\n", mHashAlgorithm.get()));
+  }
+
   nsXPIDLString errorMsg;
   PRBool noProxy = PR_TRUE;
   rv = enigmailSvc->EncryptMessageStart(nsnull, prompter,
@@ -511,7 +539,7 @@ nsEnigMsgCompose::RequiresCryptoEncapsulation(
     return rv;
 
   if (!securityInfo) {
-    *aRequiresEncryptionWork = PR_FALSE;
+    *aRequiresEncryptionWork = PR_TRUE;
     return NS_OK;
   }
 
@@ -535,9 +563,14 @@ nsEnigMsgCompose::RequiresCryptoEncapsulation(
   return NS_OK;
 }
 
+
 NS_IMETHODIMP
 nsEnigMsgCompose::BeginCryptoEncapsulation(
+#ifdef _ENIG_MOZILLA_1_8
                                         nsOutputFileStream* aStream,
+#else
+                                        nsIOutputStream* aStream,
+#endif
                                         const char* aRecipients,
                                         nsIMsgCompFields* aCompFields,
                                         nsIMsgIdentity* aIdentity,
@@ -863,11 +896,13 @@ nsEnigMsgCompose::WriteCopy(const char *aBuf, PRInt32 aLen)
 
   if (mMimeListener) {
     // Write to listener
-    mMimeListener->Write(aBuf, aLen, nsnull, nsnull);
+    rv = mMimeListener->Write(aBuf, aLen, nsnull, nsnull);
+    if (NS_FAILED(rv)) return rv;
 
   } else if (mPipeTrans) {
     // Write to process and copy if multipart/signed
-    mPipeTrans->WriteSync(aBuf, aLen);
+    rv = mPipeTrans->WriteSync(aBuf, aLen);
+    if (NS_FAILED(rv)) return rv;
 
     if (mMultipartSigned) {
       rv = WriteOut(aBuf, aLen);
