@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.94.2.0";
-var gEnigmimeVersion = "0.94.2.0";
+var gEnigmailVersion = "0.95.1";
+var gEnigmimeVersion = "0.95.1.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -63,6 +63,7 @@ const ENIG_TRANSFERABLE_CONTRACTID = "@mozilla.org/widget/transferable;1"
 const ENIG_LOCALE_SVC_CONTRACTID = "@mozilla.org/intl/nslocaleservice;1";
 const ENIG_DATE_FORMAT_CONTRACTID = "@mozilla.org/intl/scriptabledateformat;1"
 const ENIG_ACCOUNT_MANAGER_CONTRACTID = "@mozilla.org/messenger/account-manager;1";
+const ENIG_XPCOM_APPINFO = "@mozilla.org/xre/app-info;1";
 
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                               "@mozilla.org/network/file-output-stream;1";
@@ -108,13 +109,11 @@ const ENIG_TEMPDIR_PROP = "TmpD";
 var gUsePGPMimeOptionList = ["usePGPMimeNever", "usePGPMimePossible",
                              "usePGPMimeAlways"];
 
-var gEnigRecipientsSelectionOptions = ["askRecipientsNever",
-                                       "askRecipientsClever",
-                                       "askRecipientsAlways"];
-
-var gEnigPerRecipientRules = ["perRecipientRulesNo",
-                              "perRecipientRulesManual",
-                              "perRecipientRulesAlways"];
+var gEnigRecipientsSelection = ["-",
+                                "perRecipientRules",
+                                "perRecipientRulesAndEmail",
+                                "perEmailAddress",
+                                "askRecipientsAlways"];
 
 const ENIG_BUTTON_POS_0           = 1;
 const ENIG_BUTTON_POS_1           = 1 << 8;
@@ -259,55 +258,116 @@ function GetEnigmailSvc() {
 }
 
 
-function EnigUpdatePre0_8() {
+
+function EnigUpgradeRecipientsSelection () {
+  // Upgrade perRecipientRules and recipientsSelectionOption to
+  // new recipientsSelection
+
+  var  keySel = EnigGetPref("recipientsSelectionOption");
+  var  perRecipientRules = EnigGetPref("perRecipientRules");
+
+  var setVal = 2;
+
+  /*
+  1: rules only
+  2: rules & email addresses (normal)
+  3: email address only (no rules)
+  4: manually (always prompt, no rules)
+  5: no rules, no key selection
+  */
+
+  switch (perRecipientRules) {
+  case 0:
+    switch (keySel) {
+    case 0:
+      setVal = 5;
+      break;
+    case 1:
+      setVal = 3;
+      break;
+    case 2:
+      setVal = 4;
+      break;
+    default:
+      setVal = 2;
+    }
+    break;
+  case 1:
+    setVal = 2;
+    break;
+  case 2:
+    setVal = 1;
+    break;
+  default:
+    setVal = 2;
+  }
+
+  // set new pref
+  EnigSetPref("recipientsSelection", setVal);
+
+  // clear old prefs
+  gPrefEnigmail.clearUserPref("perRecipientRules");
+  gPrefEnigmail.clearUserPref("recipientsSelectionOption");
+}
+
+function EnigUpgradeHeadersView() {
+  // all headers hack removed -> make sure view is correct
+  var hdrMode = null;
   try {
-    var oldVer=EnigGetPref("configuredVersion");
+    var hdrMode = EnigGetPref("show_headers");
+  }
+  catch (ex) {}
 
-    if (oldVer.substring(0,1)=="0"){
-      if ((oldVer.substring(0,4)<"0.89") && (navigator.vendor=="Thunderbird")) {
-        // uninstall globally installed enigmime on Thunderbird
-        var ioService = enigGetService("@mozilla.org/network/io-service;1", "nsIIOService");
-        var dirService = enigGetService("@mozilla.org/file/directory_service;1", "nsIProperties");
-        var sysCompDir = dirService.get("ComsD", ENIG_C.interfaces.nsIFile);
+  if (hdrMode == null) hdrMode = 1;
+  try {
+    gPrefEnigmail.clearUserPref("show_headers");
+  }
+  catch (ex) {}
 
-        for (var f in [ "enigmime.xpt", "libenigmime.so", "enigmail.dll", "libenigmime.dylib" ]) {
-          var compFile = sysCompDir.clone();
-          compFile.append(f);
-          try {
-            if (compFile.exists()) compFile.remove(false);
-          }
-          catch (ex) {}
+  gEnigPrefRoot.setIntPref("mail.show_headers", hdrMode);
+  try {
+    enigMessageReload(false);
+  }
+  catch (ex) {}
+}
+
+function EnigUpgradePgpMime() {
+  var pgpMimeMode = false;
+  try {
+    var pgpMimeMode = (EnigGetPref("usePGPMimeOption") == 2);
+  }
+  catch (ex) {
+    return;
+  }
+
+  try {
+    if (pgpMimeMode) {
+      var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"].getService(Components.interfaces.nsIMsgAccountManager);
+      for (var i=0; i < accountManager.allIdentities.Count(); i++) {
+        var id = accountManager.allIdentities.QueryElementAt(i, Components.interfaces.nsIMsgIdentity);
+        if (id.getBoolAttribute("enablePgp")) {
+          id.setBoolAttribute("pgpMimeMode", true);
         }
       }
-      if (oldVer.substring(0,4)<"0.83") {
-        var keySrv = EnigGetPref("keyserver");
-        if (keySrv.indexOf(",") == -1) {
-          try {
-            var newKeySrv = EnigGetDefaultPref("keyserver").replace(keySrv, "");
-            newKeySrv = newKeySrv.replace(/(^, |, $)/, "").replace(/, , /,", ");
-            EnigSetPref("keyserver", keySrv+", "+newKeySrv);
-          }
-          catch (ex) {}
-        }
-      }
-      if (oldVer.substring(0,4)<"0.81") {
-        EnigOpenSetupWizard();
-      }
     }
+    gPrefEnigmail.clearUserPref("usePGPMimeOption");
   }
   catch (ex) {}
 }
 
 function EnigConfigure() {
-  try {
-    // Updates for specific versions (to be cleaned-up periodically)
-    EnigUpdatePre0_8();
-  } catch (ex) {}
-
   var oldVer=EnigGetPref("configuredVersion");
   if (oldVer == "") {
     EnigOpenSetupWizard();
   }
+  else if (oldVer < "0.95") {
+    try {
+      EnigUpgradeHeadersView();
+      EnigUpgradePgpMime();
+      EnigUpgradeRecipientsSelection();
+    }
+    catch (ex) {}
+  }
   EnigSetPref("configuredVersion", gEnigmailVersion);
   EnigSavePrefs();
 }
@@ -466,6 +526,16 @@ function CONSOLE_LOG(str) {
     gEnigmailSvc.console.write(str);
 }
 
+// write exception information
+function EnigWriteException(referenceInfo, ex) {
+  ERROR_LOG(referenceInfo+": caught exception: "
+            +ex.name+"\n"
+            +"Message: '"+ex.message+"'\n"
+            +"File:    "+ex.fileName+"\n"
+            +"Line:    "+ex.lineNumber+"\n"
+            +"Stack:   "+ex.stack+"\n");
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 function EnigAlert(mesg) {
@@ -593,13 +663,22 @@ function EnigOverrideAttribute(elementIdList, attrName, prefix, suffix) {
 
 function EnigPrefWindow(showBasic, clientType, selectTab) {
   DEBUG_LOG("enigmailCommon.js: EnigPrefWindow\n");
-  window.openDialog("chrome://enigmail/content/pref-enigmail.xul",
-                    "_blank", "chrome,resizable=yes",
-                    {'showBasic': showBasic,
-                     'clientType': clientType,
-                     'selectTab': selectTab});
-}
 
+  if (showBasic && clientType == "seamonkey" && selectTab==null) {
+    // Open the seamonkey pref window
+    goPreferences("securityItem",
+                  "chrome://enigmail/content/pref-enigmail.xul",
+                  "enigmail");
+  }
+  else {
+    // open the normal pref window
+    window.openDialog("chrome://enigmail/content/pref-enigmail.xul",
+                      "_blank", "chrome,resizable=yes",
+                      {'showBasic': showBasic,
+                      'clientType': clientType,
+                      'selectTab': selectTab});
+  }
+}
 
 function EnigAdvPrefWindow() {
   EnigAlert("This function doesn't exist anymore!");
@@ -621,19 +700,6 @@ function EnigUpgrade() {
   window.openDialog("http://enigmail.mozdev.org/no_wrap/update.html?upgrade=yes&enigmail="+gEnigmailVersion+"&enigmime="+gEnigmimeVersion, "dialog");
 }
 
-function EnigShowHeadersAll(status) {
-  DEBUG_LOG("enigmailCommon.js: EnigShowHeadersAll: "+status+"\n");
-
-  if (status && EnigGetPref("parseAllHeaders")) {
-    // Show all mail headers
-    gEnigPrefRoot.setIntPref("mail.show_headers", 2);
-
-  } else {
-    // Reset mail.show_headers pref to "original" value
-    gEnigPrefRoot.setIntPref("mail.show_headers",
-                             EnigGetPref("show_headers"));
-  }
-}
 
 function EnigDisplayRadioPref(prefName, prefValue, optionElementIds) {
   DEBUG_LOG("enigmailCommon.js: EnigDisplayRadioPref: "+prefName+", "+prefValue+"\n");
@@ -865,47 +931,24 @@ function EnigConvertToUnicode(text, charset) {
   }
 }
 
-
 function EnigConvertGpgToUnicode(text) {
   if (typeof(text)=="string") {
-    var a=text.search(/[\x80-\xFF]{2}/);
-    var b=0;
-
+    text = text.replace(/\\x3a/ig, "\\e3A");
+    a=text.search(/\\x[0-9a-fA-F]{2}/);
     while (a>=0) {
-      var ch=text.substr(a,2).toSource().substr(13,8).replace(/\\x/g, "\\u00");
-      var newCh=EnigConvertToUnicode(EnigConvertToUnicode(ch, "x-u-escaped"), "utf-8");
-      if (newCh != ch) {
-        var r=new RegExp(text.substr(a, 2), "g");
-        text=text.replace(r, newCh);
-      }
-      b=a+2;
-      a=text.substr(b+2).search(/[\x80-\xFF]{2}/);
-      if (a>=0) {
-        a += b+2;
-      }
+        ch=unescape('%'+text.substr(a+2,2));
+        r= new RegExp("\\"+text.substr(a,4));
+        text=text.replace(r, ch);
+
+        a=text.search(/\\x[0-9a-fA-F]{2}/);
     }
 
-    a=text.search(/\\x3a/i);
-    if (a>0) {
-      text = text.replace(/\\x3a/ig, "\\e3A");
-    }
-
-    a=text.search(/\\x/);
-    while (a>=0) {
-      ch=text.substr(a,4).replace(/\\x/g, "\\u00");
-      newCh=EnigConvertToUnicode(ch, "x-u-escaped");
-      if (newCh != ch) {
-        r=new RegExp("\\"+text.substr(a, 4), "g");
-        text=text.replace(r, newCh);
-      }
-      a=text.search(/\\x/);
-    }
+    text = EnigConvertToUnicode(text, "utf-8");
   }
 
   return text;
 }
 
-
 function EnigFormatFpr(fingerprint) {
   // format key fingerprint
   DEBUG_LOG("enigmailCommon.js: EnigFormatFpr: fingerprint="+fingerprint+"\n");
@@ -1261,10 +1304,7 @@ function EnigGetTempDir() {
   }
   catch (ex) {
     // let's guess ...
-    var httpHandler = ioServ.getProtocolHandler("http");
-    httpHandler = httpHandler.QueryInterface(ENIG_C.interfaces.nsIHttpProtocolHandler);
-    isWin = (httpHandler.platform.search(/Win/i) == 0);
-    if (isWin) {
+    if (EnigGetOS() == "WINNT") {
       tmpDir="C:\\TEMP";
     } else {
       tmpDir="/tmp";
@@ -1273,6 +1313,13 @@ function EnigGetTempDir() {
   return tmpDir;
 }
 
+// get the OS platform
+function EnigGetOS () {
+
+  var xulAppinfo = ENIG_C.classes[ENIG_XPCOM_APPINFO].getService(ENIG_C.interfaces.nsIXULRuntime);
+  return xulAppinfo.OS;
+
+}
 
 function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
   DEBUG_LOG("enigmailCommon.js: EnigDisplayPrefs\n");
@@ -1299,6 +1346,10 @@ function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
       switch (prefType) {
       case gPrefEnigmail.PREF_BOOL:
         if (showPrefs) {
+          if (prefElement.getAttribute("invert") == "true") {
+            prefValue = ! prefValue;
+          }
+
           if (prefValue) {
             prefElement.setAttribute("checked", "true");
           } else {
@@ -1307,10 +1358,20 @@ function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
         }
 
         if (setPrefs) {
-          if (prefElement.checked) {
-            EnigSetPref(prefName, true);
-          } else {
-            EnigSetPref(prefName, false);
+
+          if (prefElement.getAttribute("invert") == "true") {
+            if (prefElement.checked) {
+              EnigSetPref(prefName, false);
+            } else {
+              EnigSetPref(prefName, true);
+            }
+          }
+          else {
+            if (prefElement.checked) {
+              EnigSetPref(prefName, true);
+            } else {
+              EnigSetPref(prefName, false);
+            }
           }
         }
 
@@ -1539,12 +1600,12 @@ function EnigLoadKeyList(refresh, keyListObj) {
           listRow[USER_ID] = "-";
         }
         if (typeof(keyObj.userId) != "string") {
-          keyObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":"));
+          keyObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID]).replace(/\\e3A/g, ":");
           keyListObj.keySortList.push({userId: keyObj.userId, keyId: keyObj.keyId});
         }
         else {
           var subUserId = {
-            userId: EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":")),
+            userId: EnigConvertGpgToUnicode(listRow[USER_ID]).replace(/\\e3A/g, ":"),
             keyTrust: listRow[KEY_TRUST],
             type: "uid"
           }
@@ -1752,3 +1813,22 @@ function enigGetService (aURL, aInterface)
 
   return null;
 }
+
+function EnigCollapseAdvanced(obj, attribute, dummy) {
+  DEBUG_LOG("enigmailCommon.js: EnigCollapseAdvanced: test\n");
+
+  var advancedUser = EnigGetPref("advancedUser");
+
+  var obj = obj.firstChild;
+  while (obj) {
+    if (obj.getAttribute("advanced")) {
+      if (advancedUser) {
+        obj.removeAttribute(attribute);
+      }
+      else {
+        obj.setAttribute(attribute, "true");
+      }
+    }
+    obj = obj.nextSibling;
+  }
+}
\ No newline at end of file
