@@ -49,7 +49,7 @@ const EnigOutputLFLineBreak   = 1024;
 const ENIG_ENIGMSGCOMPFIELDS_CONTRACTID = "@mozdev.org/enigmail/composefields;1";
 
 // List of hash algorithms for PGP/MIME signatures
-var gMimeHashAlgorithms = ["md5", "sha1", "ripemd160", "sha256", "sha384", "sha512"];
+var gMimeHashAlgorithms = [null, "sha1", "ripemd160", "sha256", "sha384", "sha512"];
 
 var gEnigEditor;
 var gEnigDirty, gEnigProcessed, gEnigTimeoutID;
@@ -154,6 +154,10 @@ function enigGetAccDefault(value) {
       break;
     case 'signEnc':
       r=gEnigIdentity.getBoolAttribute("pgpSignEncrypted");
+      break;
+    case 'pgpMimeMode':
+      r=gEnigIdentity.getBoolAttribute("pgpMimeMode");
+      break;
     }
     DEBUG_LOG("  "+value+"="+r+"\n");
     return r;
@@ -164,6 +168,7 @@ function enigGetAccDefault(value) {
       return 0;
     case 'signPlain':
     case 'signEnc':
+    case 'pgpMimeMode':
       return false;
     }
   }
@@ -199,6 +204,8 @@ function enigSetSendDefaultOptions() {
   else {
     if (enigGetAccDefault("signPlain")) gEnigSendMode |= ENIG_SIGN;
   }
+
+  gEnigSendPGPMime = enigGetAccDefault("pgpMimeMode");
 }
 
 function enigRemoveAttachedSig() {
@@ -208,6 +215,12 @@ function enigRemoveAttachedSig() {
 
 function enigComposeOpen() {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigComposeOpen\n");
+
+  var toobarElem = document.getElementById("composeToolbar2");
+  if (toobarElem && (EnigGetOS() == "Darwin")) {
+    toobarElem.setAttribute("platform", "macos");
+  }
+
   if (EnigGetPref("keepSettingsForReply") && (!(gEnigSendMode & ENIG_ENCRYPT))) {
     var enigMimeService = Components.classes[ENIG_ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
     if (enigMimeService)
@@ -240,7 +253,6 @@ function enigMsgComposeReopen() {
    DEBUG_LOG("enigmailMsgComposeOverlay.js: enigMsgComposeReopen\n");
    enigMsgComposeReset();
    enigComposeOpen();
-
 }
 
 function enigMsgComposeClose() {
@@ -261,11 +273,6 @@ function enigMsgComposeReset() {
   gEnigEnableRules = true;
   gEnigIdentity = null;
 
-  EnigShowHeadersAll(true);
-
-  gEnigSendPGPMime = !(EnigGetPref("usePGPMimeOption") == PGP_MIME_ALWAYS);
-  enigTogglePGPMime();
-
   enigSetIdentityDefaults();
 }
 
@@ -366,7 +373,9 @@ function enigExtractAndAttachKey(uid) {
       return;
     }
   }
-  catch (ex) {}
+  catch (ex) {
+    EnigWriteException("enigmailMsgComposeOverlay.js: enigExtractAndAttachKey", ex);
+  }
   tmpFile.append("key.asc");
   tmpFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
 
@@ -685,6 +694,7 @@ function enigConfirmBeforeSend(toAddr, gpgKeys, sendFlags, isOffline, msgSendTyp
   return EnigConfirm(msgConfirm);
 }
 
+
 function enigAddRecipients(toAddrList, recList) {
   for (var i=0; i<recList.count; i++) {
     toAddrList.push(EnigStripEmail(recList.StringAt(i).replace(/[\",]/g, "")));
@@ -716,6 +726,20 @@ function enigEncryptMsg(msgSendType) {
     break;
   }
 
+  var msgCompFields = gMsgCompose.compFields;
+  var newsgroups = msgCompFields.newsgroups;  // Check if sending to any newsgroups
+
+  if ((! (sendFlags & nsIEnigmail.SAVE_MESSAGE)) &&
+      msgCompFields.to == "" &&
+      msgCompFields.cc == "" &&
+      msgCompFields.bcc == "" &&
+      newsgroups == "") {
+    // don't attempt to send message if no recipient specified
+    EnigAlert(sComposeMsgsBundle.getString("12511"));
+    window.cancelSendMessage=true;
+    return;
+  }
+
   if (gotSendFlags & ENIG_SIGN)
       sendFlags |= ENIG_SIGN;
   if (gotSendFlags & ENIG_ENCRYPT) {
@@ -735,7 +759,6 @@ function enigEncryptMsg(msgSendType) {
     return;
   }
 
-
   if (gEnigDirty) {
     // make sure the sendFlags are reset before the message is processed
     // (it may have been set by a previously cancelled send operation!)
@@ -757,7 +780,9 @@ function enigEncryptMsg(msgSendType) {
           gMsgCompose.compFields.securityInfo = newSecurityInfo;
         }
       }
-      catch (ex) {}
+      catch (ex) {
+        EnigWriteException("enigmailMsgComposeOverlay.js: enigAttachKey", ex);
+      }
     }
   }
   gEnigDirty = true;
@@ -795,7 +820,7 @@ function enigEncryptMsg(msgSendType) {
         return;
      }
 
-     var recipientsSelectionOption = EnigGetPref("recipientsSelectionOption");
+     var recipientsSelection = EnigGetPref("recipientsSelection");
 
      var optSendFlags = 0;
      var inlineEncAttach=false;
@@ -834,10 +859,10 @@ function enigEncryptMsg(msgSendType) {
        }
      }
 
-     var msgCompFields = gMsgCompose.compFields;
-
-     // Check if sending to any newsgroups
-     var newsgroups = msgCompFields.newsgroups;
+     if (typeof(userIdValue) != "string") {
+       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: type of userIdValue="+typeof(userIdValue)+"\n");
+       userIdValue = "";
+     }
 
      DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg:gMsgCompose="+gMsgCompose+"\n");
 
@@ -922,10 +947,10 @@ function enigEncryptMsg(msgSendType) {
      if (toAddr.length>=1) {
 
         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: toAddr="+toAddr+"\n");
-        var perRecipientRules=EnigGetPref("perRecipientRules");
         var repeatSelection=0;
         while (repeatSelection<2) {
-          if (perRecipientRules>0 && gEnigEnableRules) {
+          if (recipientsSelection != 3 && recipientsSelection != 4
+              && gEnigEnableRules) {
             var matchedKeysObj = new Object;
             var flagsObj=new Object;
             if (!getRecipientsKeys(toAddr,
@@ -976,33 +1001,31 @@ function enigEncryptMsg(msgSendType) {
 
             var testPlain = "Test Message";
             var testUiFlags   = nsIEnigmail.UI_TEST;
-            var testSendFlags = nsIEnigmail.SEND_ENCRYPTED |
-                                nsIEnigmail.SEND_TEST |
+            var testSendFlags = nsIEnigmail.SEND_TEST | ENIG_ENCRYPT |
                                 optSendFlags;
 
             // test recipients
-            testCipher = enigmailSvc.encryptMessage(window, testUiFlags,
-                                                          testPlain,
-                                                          fromAddr, toAddr,
-                                                          testSendFlags,
-                                                          testExitCodeObj,
-                                                          testStatusFlagsObj,
-                                                          testErrorMsgObj);
+            testCipher = enigmailSvc.encryptMessage(window, testUiFlags, null,
+                                                    testPlain,
+                                                    fromAddr, toAddr,
+                                                    testSendFlags,
+                                                    testExitCodeObj,
+                                                    testStatusFlagsObj,
+                                                    testErrorMsgObj);
 
-
-            if ((recipientsSelectionOption==2) ||
+            if ((recipientsSelection==4) ||
                 ((testStatusFlagsObj.value & nsIEnigmail.INVALID_RECIPIENT) &&
-                 (recipientsSelectionOption>0))) {
+                 (recipientsSelection==2 || recipientsSelection==3))) {
                 var resultObj = new Object();
                 var inputObj = new Object();
                 inputObj.toAddr = toAddr;
                 inputObj.invalidAddr = enigGetInvalidAddress(testErrorMsgObj.value);
                 inputObj.options = "multisel";
-                if (perRecipientRules<2)
+                if (recipientsSelection==2)
                   inputObj.options += ",rulesOption"
                 if (notSignedIfNotEnc)
                   inputObj.options += ",notsigned";
-                if (recipientsSelectionOption<2)
+                if (recipientsSelection == 4)
                   inputObj.options += ",noforcedisp";
                 inputObj.dialogHeader = EnigGetString("recipientsSelectionHdr");
 
@@ -1031,7 +1054,7 @@ function enigEncryptMsg(msgSendType) {
                   return;
                 }
             }
-            if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelectionOption==0) {
+            if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelection==5) {
                 // Test encryption failed; turn off default encryption
                 sendFlags &= ~ENIG_ENCRYPT;
                 DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: No default encryption because test failed\n");
@@ -1065,8 +1088,7 @@ function enigEncryptMsg(msgSendType) {
         enigmailSvc.composeSecure) {
 
         inputObj = new Object();
-        inputObj.pgpMimePossible = (usePGPMimeOption >= PGP_MIME_POSSIBLE);
-        // inputObj.inlinePossible = (sendFlags & ENIG_ENCRYPT); // makes no sense for sign only!
+        inputObj.pgpMimePossible = true;
         inputObj.inlinePossible = true;
         inputObj.restrictedScenario = false;
 
@@ -1132,6 +1154,18 @@ function enigEncryptMsg(msgSendType) {
        usingPGPMime = false;
      }
 
+     // Detect PGP/MIME and S/MIME
+     if (usingPGPMime) {
+        if (gMsgCompose.compFields.securityInfo instanceof Components.interfaces.nsIMsgSMIMECompFields) {
+            if (gMsgCompose.compFields.securityInfo.requireEncryptMessage ||
+              gMsgCompose.compFields.securityInfo.signMessage) {
+                EnigAlert("pgpMime.sMime.incomaptible");
+                window.cancelSendMessage=true;
+                return;
+            }
+        }
+     }
+
      var uiFlags = nsIEnigmail.UI_INTERACTIVE;
 
      if (usingPGPMime)
@@ -1218,12 +1252,6 @@ function enigEncryptMsg(msgSendType) {
          ((sendFlags & ENIG_ENCRYPT_OR_SIGN))) {
       // temporarily enable quoted-printable for PGP/MIME messages
        try {
-          if (! gEnigPrefRoot.getBoolPref("mail.strictly_mime")) {
-            gEnigPrefRoot.setBoolPref("mail.strictly_mime", true);
-            newSecurityInfo.UIFlags |= nsIEnigmail.UI_RESTORE_STRICTLY_MIME;
-            DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled quoted-printable\n");
-          }
-
           // make sure plaintext is not changed to 7bit
           if (typeof(msgCompFields.forceMsgEncoding) == "boolean") {
             msgCompFields.forceMsgEncoding = true;
@@ -1241,6 +1269,7 @@ function enigEncryptMsg(msgSendType) {
     }
 
   } catch (ex) {
+     EnigWriteException("enigmailMsgComposeOverlay.js: enigEncryptMsg", ex);
      msg=EnigGetString("signFailed");
      if (gEnigmailSvc && gEnigmailSvc.initializationError) {
         msg += "\n"+gEnigmailSvc.initializationError;
@@ -1268,7 +1297,9 @@ function enigEncryptInline(sendInfo) {
         return false;
       }
     }
-  } catch (ex) {}
+  } catch (ex) {
+     EnigWriteException("enigmailMsgComposeOverlay.js: enigEncryptInline", ex);
+  }
 
   try {
     if (gEnigPrefRoot.getBoolPref("mail.strictly_mime")) {
@@ -1373,7 +1404,7 @@ function enigEncryptInline(sendInfo) {
                    ? EnigConvertFromUnicode(origText, charset)
                    : EnigConvertFromUnicode(escText, charset);
 
-    var cipherText = enigmailSvc.encryptMessage(window, sendInfo.uiFlags, plainText,
+    var cipherText = enigmailSvc.encryptMessage(window, sendInfo.uiFlags, null, plainText,
                                           sendInfo.fromAddr, sendInfo.toAddr, sendInfo.sendFlags,
                                           exitCodeObj, statusFlagsObj,
                                           errorMsgObj);
@@ -1497,7 +1528,9 @@ function enigModifyCompFields(msgCompFields) {
       msgCompFields.otherRandomHeaders += enigmailHeaders;
     }
   }
-  catch (ex) {}
+  catch (ex) {
+    EnigWriteException("enigmailMsgComposeOverlay.js: enigModifyCompFields", ex)
+  }
 
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigModifyCompFields: otherRandomHeaders = "+
            msgCompFields.otherRandomHeaders+"\n");
@@ -1544,15 +1577,7 @@ function enigGenericSendMessage( msgType )
         {
           if (gEnigPromptSvc)
           {
-            var msgTitle;
-            try {
-              // TB 1.1
-              msgTitle = sComposeMsgsBundle.getString("sendMsgTitle");
-            }
-            catch (ex) {
-              // TB <= 1.0
-              msgTitle = sComposeMsgsBundle.getString("subjectDlogTitle");
-            }
+            var msgTitle = sComposeMsgsBundle.getString("sendMsgTitle");
             var result = {value:sComposeMsgsBundle.getString("defaultSubject")};
             if (gEnigPromptSvc.prompt(
                         window,
@@ -1722,10 +1747,19 @@ function enigGenericSendMessage( msgType )
         var event = document.createEvent('Events');
         event.initEvent('compose-send-message', false, true);
         document.getElementById("msgcomposeWindow").dispatchEvent(event);
-
-        gWindowLocked = true;
-        disableEditableFields();
-        updateComposeItems();
+        gAutoSaving = (msgType == nsIMsgCompDeliverMode.AutoSaveAsDraft);
+        // disable the ui if we're not auto-saving
+        if (!gAutoSaving)
+        {
+          gWindowLocked = true;
+          disableEditableFields();
+          updateComposeItems();
+        }
+        // if we're auto saving, mark the body as not changed here, and not
+        // when the save is done, because the user might change it between now
+        // and when the save is done.
+        else
+          SetContentAndBodyAsUnmodified();
 
         var progress = Components.classes["@mozilla.org/messenger/progress;1"].createInstance(Components.interfaces.nsIMsgProgress);
         if (progress)
@@ -1733,7 +1767,14 @@ function enigGenericSendMessage( msgType )
           progress.registerListener(progressListener);
           gSendOrSaveOperationInProgress = true;
         }
-        msgWindow.SetDOMWindow(window);
+
+        try {
+          // TB <= 2.0
+          msgWindow.SetDOMWindow(window);
+        }
+        catch (ex) {
+          msgWindow.domWindow = window;
+        }
         msgWindow.rootDocShell.allowAuth = true;
         gMsgCompose.SendMsg(msgType, getCurrentIdentity(), currentAccountKey, msgWindow, progress);
 /*
@@ -1768,8 +1809,6 @@ function enigGenericSendMessage( msgType )
 
 function enigCheckCharsetConversion(msgCompFields) {
 
-  if (msgCompFields.characterSet == "UTF-8") return true;
-
   try {
     var encoderFlags = EnigOutputFormatted | EnigOutputLFLineBreak;
     var docText = EnigEditorGetContentsAs("text/plain", encoderFlags);
@@ -2057,12 +2096,14 @@ function enigDecryptQuote(interactive) {
   var keyIdObj       = new Object();
   var sigDateObj     = new Object();
   var errorMsgObj    = new Object();
+  var blockSeparationObj  = new Object();
 
   var uiFlags = nsIEnigmail.UI_UNVERIFIED_ENC_OK;
 
   var plainText = enigmailSvc.decryptMessage(window, uiFlags, cipherText,
                                  signatureObj, exitCodeObj, statusFlagsObj,
-                                 keyIdObj, userIdObj, sigDateObj, errorMsgObj);
+                                 keyIdObj, userIdObj, sigDateObj,
+                                 errorMsgObj, blockSeparationObj);
 
   // Decode plaintext from charset to unicode
   plainText = EnigConvertToUnicode(plainText, charset);
